"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const chalk = require("chalk");
const dateFns = require("date-fns");
const pathHelper_1 = require("./../helpers/pathHelper");
const utils_1 = require("./../helpers/utils");
const envHelper_1 = require("../helpers/envHelper");
const config_1 = require("../config/config");
const packagesHelper_1 = require("../helpers/packagesHelper");
const install_1 = require("../scripts/install");
const build_1 = require("../scripts/build");
exports.default = {
    command: 'deploy [command]',
    describe: 'Deploy project for production',
    handler: commandHandler,
    builder: commandBuilder
};
const TARGET_HEROKU = 'heroku';
const TARGET_LOCAL = 'local';
const TARGET_NOW = 'now';
const COMMAND_STOP = 'stop';
const COMMAND_INIT = 'init';
const VALID_TARGETS = [TARGET_LOCAL, TARGET_HEROKU, TARGET_NOW];
const VALID_COMMANDS = [COMMAND_STOP, COMMAND_INIT];
const HEROKU_INSTANCE = 'heroku';
function commandBuilder(yargs) {
    return yargs
        .option('stop', {
        description: 'Stop running application process for local deployment'
    })
        .option('target', {
        alias: 't',
        description: 'Deployment target, supported targets are heroku/now/local, local is deafult'
    })
        .option('heroku-app', {
        alias: 'happ',
        description: 'For heroku only, specify Heroku App ID'
    })
        .option('skip-client-build', {
        alias: 'scb',
        description: 'Skip client build'
    })
        .example('deploy', 'Deploy project for production (starts project with one of supported process managers).')
        .example('deploy --stop', 'Stop running app (it is removed from process list and cannot be restarted again).')
        .example('deploy -t heroku -happ my-heroku-app', 'Deploys app to heroku with APP_ID: "my-heroku-app".');
}
function commandHandler(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        envHelper_1.default.checkFolderStructure();
        let target = getTarget(argv);
        let appName = envHelper_1.default.getAppName();
        let processManager = null;
        if (target === TARGET_LOCAL && argv.stop) {
            processManager = detectProcessManager();
            return utils_1.default.logOperation(`Stop '${appName}' application process`, () => {
                stopLocalApp(processManager, appName);
            });
        }
        let deployDir = pathHelper_1.default.projectRelative(config_1.default.paths.deploy.root, target);
        let buildDir = pathHelper_1.default.projectRelative(config_1.default.paths.build.root);
        let deployParams = {
            target,
            appName,
            deployDir,
            buildDir,
            processManager,
            argv
        };
        try {
            utils_1.default.ensureEmptyDir(deployDir);
            beforeDeploy(deployParams);
            yield ensureBuild(argv.skipClientBuild);
            utils_1.default.logOperation('Copy build assets', () => {
                let isFirstDeploy = !fs.existsSync(deployDir);
                if (isFirstDeploy) {
                    utils_1.default.ensureEmptyDir(deployDir);
                }
                else {
                    let localDir = pathHelper_1.default.deployRelative(config_1.default.paths.server.local);
                    let gitDir = `!${deployDir}/.git`;
                    let ignoreList = [localDir, gitDir];
                    if (target === TARGET_NOW) {
                        ignoreList.push('now.json');
                    }
                    utils_1.default.clearDir(deployDir, ignoreList);
                }
                fs.copySync(buildDir, deployDir);
            });
            afterDeploy(deployParams);
        }
        catch (err) {
            utils_1.default.logAndExit(err);
        }
    });
}
function getTarget(argv) {
    let target = TARGET_LOCAL;
    if (argv.target) {
        utils_1.default.assertValueIsInTheList(argv.target, VALID_TARGETS, `Invalid deploy target.`);
        target = argv.target;
        if (target === TARGET_HEROKU) {
            if (!argv.herokuApp) {
                utils_1.default.logAndExit('Specify heroku APP ID for heroku deployments');
            }
        }
    }
    return target;
}
function beforeDeploy(deployParams) {
    const { target, processManager, appName, deployDir } = deployParams;
    switch (target) {
        case TARGET_LOCAL:
            stopLocalApp(processManager, appName);
            break;
        case TARGET_HEROKU:
            beforeDeployHeroku(deployParams);
            break;
        case TARGET_NOW:
            let nowConfig = getNowConfig(deployDir);
            let nowAppName = nowConfig.name;
            if (!nowAppName)
                utils_1.default.logAndExit('Specify now deployment name in now.json config.');
            utils_1.default.runCommand('now', ['rm', nowAppName, '-y'], {
                title: `Remove previous now deployments`,
                path: deployDir
            });
            break;
    }
}
function beforeDeployHeroku(deployParams) {
    const { deployDir, argv } = deployParams;
    const remoteHasMaster = () => {
        let output = utils_1.default.getCommandOutput('git', ['ls-remote', '--heads', HEROKU_INSTANCE, 'master'], deployDir);
        return output.output ? true : false;
    };
    const gitCommand = (args, title = '', allowError = false) => utils_1.default.runCommand('git', args, {
        title,
        path: deployDir,
        ignoreError: true
    });
    gitCommand(['init']);
    gitCommand(['config', 'user.email', 'napp.heroku@gmail.com']);
    gitCommand(['config', 'user.name', 'Napp Deploy']);
    const herokuAppId = argv.herokuApp;
    if (!herokuAppId) {
        utils_1.default.logAndExit('Specify Heroku App ID (--heroku-app) for initial deployment (see "deploy" command help for more information)');
    }
    utils_1.default.runCommand('heroku', ['git:remote', '-a', herokuAppId, '-r', HEROKU_INSTANCE], {
        title: `Init heroku remote "${HEROKU_INSTANCE}"`,
        path: deployDir
    });
    if (remoteHasMaster()) {
        gitCommand(['fetch', HEROKU_INSTANCE]);
        gitCommand(['checkout', `${HEROKU_INSTANCE}/master`, '-b', HEROKU_INSTANCE]);
    }
}
function afterDeploy(deployParams) {
    const { target, processManager, appName, deployDir } = deployParams;
    switch (target) {
        case TARGET_LOCAL:
            //install packages there
            let installCommandInfo = packagesHelper_1.default.getInstallPackagesCommand();
            utils_1.default.runCommand(installCommandInfo.command, installCommandInfo.params, {
                title: 'Install production dependencies',
                path: deployParams.deployDir
            });
            startLocalApp(processManager, appName);
            break;
        case TARGET_HEROKU:
            utils_1.default.runCommand('git', ['checkout', '-b', HEROKU_INSTANCE], {
                path: deployDir,
                ignoreError: true
            });
            utils_1.default.runCommand('git', ['add', '.'], {
                title: 'Add files to git',
                path: deployDir
            });
            utils_1.default.runCommand('git', ['commit', '-m', `"Deployment at ${dateFns.format(new Date(), 'YYYY-MM-DDTHH:mm:ss')}"`], {
                title: 'Commit files to git',
                userError: 'It seems that you have deployed the current code already. Make some changes and try again.',
                path: deployDir
            });
            utils_1.default.runCommand('git', ['push', HEROKU_INSTANCE, `${HEROKU_INSTANCE}:master`], {
                title: 'Deploying to Heroku...',
                showOutput: true,
                path: deployDir
            });
            break;
        case TARGET_NOW:
            utils_1.default.runCommand('now', ['-y', '--public'], {
                path: deployDir,
                title: 'Deploy to Now',
                showOutput: true
            });
            let nowConfig = getNowConfig(deployDir);
            if (nowConfig.alias) {
                utils_1.default.runCommand('now', ['alias'], {
                    path: deployDir,
                    title: 'Add alias',
                    showOutput: true
                });
            }
            break;
    }
}
function ensureBuild(skipClientBuild) {
    //return Promise.resolve(null);
    install_1.default.installAll();
    return build_1.default.build(skipClientBuild ? 'server' : 'full');
}
function detectProcessManager() {
    let processManager = utils_1.default.findGlobalCommandByPrecedence(['pm2', 'forever']);
    if (!processManager) {
        utils_1.default.logAndExit(`Install globally one of supported process managers: forever or pm2.`);
    }
    return processManager;
}
function stopLocalApp(processManager, appName) {
    let params = [];
    if (processManager === 'forever') {
        params = ['stop', appName];
    }
    if (processManager === 'pm2') {
        params = ['delete', appName];
    }
    utils_1.default.runCommand(processManager, params, {
        path: getLocalDeploymentDir(),
        ignoreError: true,
        showOutput: false
    });
}
function startLocalApp(processManager, appName) {
    let params = [];
    if (processManager === 'forever') {
        params = ['start', '--id', appName, 'index.js'];
    }
    if (processManager === 'pm2') {
        params = ['start', 'index.js', '--name', appName];
    }
    utils_1.default.runCommand(processManager, params, {
        title: 'Start process',
        path: getLocalDeploymentDir()
    });
    utils_1.default.logAndExit(`Process has been started. By default it is available on ${chalk.cyan('http://localhost:5000')}.`);
}
function getLocalDeploymentDir() {
    return pathHelper_1.default.projectRelative(config_1.default.paths.deploy.root, TARGET_LOCAL);
}
function getNowConfig(deployDir) {
    let nowConfigPath = pathHelper_1.default.path.join(deployDir, 'now.json');
    if (!fs.existsSync(nowConfigPath))
        utils_1.default.logAndExit('Create now.json configuration file in deployment folder.');
    let nowConfig = fs.readJsonSync(nowConfigPath);
    return nowConfig;
}
//# sourceMappingURL=deploy.js.map